//======================================================================
// File:		dampedfloat.cpp
// Author:	Matthias Toussaint
// Created:	Sun Sep 22 02:35:58 CEST 2002
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2000-2002 Matthias Toussaint
//======================================================================

#include <dampedfloat.h>

#include <qglobal.h>  // QMIN

#include <iostream>

DampedFloat::DampedFloat( int depth ) :
  m_maxDepth( depth ),
  m_variance( 10 )
{
  m_array = new float [m_maxDepth];
  
  reset();
}

DampedFloat::~DampedFloat()
{
  delete [] m_array;
}

const float
DampedFloat::value() const
{
  if (0 == m_counter) return 0.0;
  
  return m_sum / (float)m_counter;
}

void
DampedFloat::reset()
{
  m_sum     = 0;
  m_counter = 0;
  m_pointer = 0;
}

void
DampedFloat::addValue( float val )
{
  if (m_maxDepth == m_counter)
  {
    m_sum -= m_array[m_pointer];
  }
  
  m_array[m_pointer] = val;
  
  ++m_pointer;
  m_pointer %= m_maxDepth;
  
  m_counter = QMIN( m_maxDepth, m_counter+1 );

  if (val != 0.0)
  {
    float fac = fabs( value() / val );
    
    if (fac < 1.2 && fac > 0.83333) m_variance--;
    else                            m_variance++;
  }
  
  m_variance = QMIN( 10, m_variance );
  m_variance = QMAX( 0, m_variance );
  
  m_sum += val;  
}

const bool
DampedFloat::valid() const
{
  return ((0 != m_counter) && (m_variance == 0));
}
