//======================================================================
// File:		simulator.h
// Author:	Matthias Toussaint
// Created:	Sat Jul  6 21:02:10 CEST 2002
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2000-2002 Matthias Toussaint
//======================================================================

#ifndef SIMULATOR_HH
#define SIMULATOR_HH

#include <qobject.h>
#include <dso.h>
#include <stdlib.h>

class Simulator : public QObject, public Dso
{
  Q_OBJECT
      
public:
  enum Waveform
  {
    Sine = 0,
    Square,
    Sawtooth
  };
    
  Simulator();
  virtual ~Simulator();
  
  void run();
  
  void setFrequency( int, float );
  void setVpp( int, float );
  
  virtual bool connectToHardware() { return true; }
  /// Timebase in us
  virtual void setTimeBase( Dso::TimeBase );
  /// volts/div in mv
  virtual void setVoltsDiv( int, Dso::VoltsDiv );

  virtual inline const int triggerBits() const { return 0xf0; }  
  virtual const bool hasTriggerFilter() const { return false; }
  virtual const int  numChannel() const { return 2; }
  virtual const int  bits() const { return 8; }
  virtual const bool hasDcOffset() const { return false; }
  
public slots:
  void waveformChangedSLOT( unsigned, Simulator::Waveform );
  void frequencyChangedSLOT( unsigned, float );
  void vppChangedSLOT( unsigned, float );
  void noiseToggledSLOT( unsigned, bool );

protected:
  float     m_freq[2];
  Waveform  m_waveform[2];
  bool      m_addNoise[2];
  float     m_vPp[2];
  float     m_threshold[2];
  float     m_samplesPerPeriod[2];
  float     m_div[2];
  float     m_a[2];
  float     m_sawtoothStep[2];
  float     m_sawtoothValue[2];
  float     m_sawtoothOff[2];
  int       m_cnt;
  unsigned *m_ringBuffer[2];
  int       m_ringBufferPointer;
  
  void setupSimulator();
  unsigned sample( int );
  bool raisingTrigger() const;
  bool fallingTrigger() const;
  
  inline float noise( const int ) const;
  inline unsigned clamp( const float ) const;
  
};

float
Simulator::noise( const int channel ) const
{
  return (m_addNoise[channel] ? 
          ((float)random()/(float)RAND_MAX*2.-1.) : 0.);
//          ((float)random()/(float)RAND_MAX-0.5) : 0.);
}

unsigned
Simulator::clamp( const float value ) const
{
  return (unsigned)qRound( QMIN( 255., QMAX( value, 0. ) ) );
}

#endif // SIMULATOR_HH
